#!/usr/bin/perl
###############################################################################
#    Webmin Sysstats - irq-lib.pl
#
#    Webmin Sysstats Module ver 0.7
#    Copyright (C) 2001 by David Bouius
#    Bug reports to: dbouius@users.sourceforge.net
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
###############################################################################

sub install_module {
  local ($minfo, $defaults) = ($_[0], $_[1]);

  local (%my_config);

  $my_config{'db_uid'} = 1;
  $my_config{'sample_rate'} = 1;

  &write_config($minfo, \%my_config, $defaults);
  &install_script($minfo, 'irq.sh');

  return 1;
}

sub remove_module {
}

sub mod_check_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  return $my_config->{'runstop'};
}

sub mod_runstop {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $my_config->{'runstop'} = $my_config->{'runstop'} ? '1' : '0';
  &write_config($minfo, $my_config, $defaults);
}

sub mod_check_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  if (-r $minfo->{'config_dir'} . "../irq/config.cfg") {
    return 1;
  }
  return 0;
}

sub mod_upgrade {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local(%old_config);

  if (-r $minfo->{'config_dir'} . "../irq/config.cfg" ) {

    my $cmd = "cp -f " . $minfo->{'dbtables'} . "../irq/*.rrd " . $minfo->{'dbtables'};
    system($cmd);

    if (!&read_file($minfo->{'config_dir'} . "../irq/config.cfg", \%old_config)) {
      &error(&text('ecannot_readconfig', $minfo->{'config_dir'} . "../irq/config.cfg"));
    }
    &write_config($minfo, \%old_config, $defaults);
  }
}

sub graph_data {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  $rrdtool = $config{'rrdtool'};
  $sizex = $config{'graph_sizex'};
  $sizey = $config{'graph_sizey'};

  $rrd_detals  = " -v 'Interrupts'";
  $rrd_detals .= " COMMENT:'Interrupts        Max     Min  Average\\n'";

  my $irq_num = 1;
  while ($my_config->{'irq' . $irq_num} ne "") {
    local ($irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display)
      = split(',', $my_config->{'irq' . $irq_num});

    if (!$display) {
      $irq_name .= substr("...........", 0, (0 - length($irq_name)));
      $rrd_detals .= " DEF:irq" . $irq_num . "=" . $minfo->{'dbtables'} . $irq_id . ".rrd:irq:MAX";
      $rrd_detals .= " LINE1:irq" . $irq_num . "#" . $irq_colour . ":" . $irq_name;
      $rrd_detals .= " GPRINT:irq" . $irq_num . ":MAX:'%6.2lf'";
      $rrd_detals .= " GPRINT:irq" . $irq_num . ":MIN:'%6.2lf'";
      $rrd_detals .= " GPRINT:irq" . $irq_num . ":AVERAGE:'%6.2lf\\n'";
    }
    $irq_num++;
  }

  foreach $graph ('hour', 'day', 'week', 'month') {
    if ($graph eq 'hour') {
      $time = "-3600";
      $title = "'The last hour'";
    } elsif ($graph eq 'day') {
      $time = "-86400";
      $title = "'The last day'";
    } elsif ($graph eq 'week') {
      $time = "-604800";
      $title = "'The last week'";
    } elsif ($graph eq 'month') {
      $time = "-18748800";
      $title = "'The last month'";
    }

    $common  = $rrdtool . " graph " . $minfo->{'tmpgname'} . "_" . $graph . ".png";
    $common .= " --start " . $time . " --end -60";
    $common .= " -h " . $sizey . " -w " . $sizex;
    $common .= " -a PNG --title=" . $title;

    $sig = " COMMENT:'\\n' COMMENT:\"Created: `date`\"";

    system($common . $rrd_detals . $sig);
  }
}

sub mod_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  if ($in{'opt'}) {
    if ($in{'opt'} eq 'up') {
      my $tmp = $my_config->{'irq' . $in{'irq_num'}};
      $my_config->{'irq' . $in{'irq_num'}} = $my_config->{'irq' . ($in{'irq_num'} - 1)};
      $my_config->{'irq' . ($in{'irq_num'} - 1)} = $tmp;
    } elsif ($in{'opt'} eq 'down') {
      my $tmp = $my_config->{'irq' . $in{'irq_num'}};
      $my_config->{'irq' . $in{'irq_num'}} = $my_config->{'irq' . ($in{'irq_num'} + 1)};
      $my_config->{'irq' . ($in{'irq_num'} + 1)} = $tmp;
    } elsif ($in{'opt'} eq 'runstop') {
      local ($irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display)
	= split(',', $my_config->{'irq' . $in{'irq_num'}});

      $runstop = $runstop ? 0 : 1;
      $my_config->{'irq' . $in{'irq_num'}}
	= join(',', $irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display);
    } elsif ($in{'opt'} eq 'display') {
      local ($irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display)
	= split(',', $my_config->{'irq' . $in{'irq_num'}});

      $display = $display ? 0 : 1;
      $my_config->{'irq' . $in{'irq_num'}}
	= join(',', $irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display);
    } elsif ($in{'opt'} eq 'rm') {
      my $irq_num = $in{'irq_num'};

      local ($irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display)
	= split(',', $my_config->{'irq' . $irq_num});

      while ($my_config->{'irq' . $irq_num} ne '') {
	$my_config->{'irq' . $irq_num++} = $my_config->{'irq' . ($irq_num)};
      }

      $my_config->{'irq' . ($irq_num - 1)} = '';

      &write_config($minfo, $my_config, $defaults);

      unlink ($minfo->{'dbtables'} . $irq_id . '.rrd');

    }

    &write_config($minfo, $my_config, $defaults);
  }
  print "<table border width='100%'>\n";
  print  "<tr $cb>";
  print   "<td>";
  print    "<table width='100%'>\n";
  print     "<tr><td><b> $text{'module_subtitle'}</b></td></tr>\n";

  print     "<tr><td><table border width='100%'>\n";
  print     "<td $tb><b>$text{'module_subtitle_irq_name'}</b></td>";
  print     "<td $tb><b>$text{'module_display_name'}</b></td>";
  print     "<td $tb><b>$text{'module_subtitle_disp_colour'}</b></td>";
  print     "<td $tb width='" . (22 * 6) . "'><b>$text{'module_subtitle_controls'}</b></td>";

  @irq_list = split ( /\n/, `cat /proc/interrupts | grep -v CPU | awk '{print \$1 \$4 \$5 \$6 \$7 \$8}'`);

  my $irq_num = 1;
  while ($my_config->{'irq' . $irq_num} ne "") {
    local ($irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display)
      = split(',', $my_config->{'irq' . $irq_num});

    for ($i=0; $i < scalar (@irq_list); $i++) {
      local ($n, $name) = split ( /:/, $irq_list[$i]);

      if ($n eq $irq_number) {
	$irq_sname = $name;
	last;
      }
    }

    print    "<tr $cb>";
    print     "<td><b><a href='edit_module.cgi?idx=$in{'idx'}&opt=edit&irq_num=$irq_num'>$irq_sname</a></b></td>";
    print     "<td>$irq_name</td>";
    print     "<td><font color='$irq_colour'><b>";
    print     $text{'colour_' . $irq_colour} ? $text{'colour_' . $irq_colour} : "<pre>#$irq_colour </pre>";
    print     "</b></font></pre></td>";
    print     "<td align='right'>";
    if ($access{'configure'}) {
      if ($irq_num > 1) {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=up&irq_num=$irq_num'>";
	print "<IMG src='images/up.png' border='0'>";
	print "</a>";
      } else {
	print "<IMG src='images/space.png' border='0'>";
	print "<IMG src='images/space.png' border='0'>";
      }
      print "<IMG src='images/space.png' border='0'>\n";
      if ($my_config->{'irq' . ($irq_num + 1)} ne '') {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=down&irq_num=$irq_num'>";
	print "<IMG src='images/down.png' border='0'>";
	print "</a>";
      } else {
	print "<IMG src='images/space.png' border='0'>";
	print "<IMG src='images/space.png' border='0'>";
      }
      print "<IMG src='images/space.png' border='0'>\n";
      if ($runstop) {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=runstop&irq_num=$irq_num'>";
	print "<IMG src='images/record.png' border='0'>";
	print "</a>";
      } else {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=runstop&irq_num=$irq_num'>";
	print "<IMG src='images/pause.png' border='0'>";
	print "</a>";
      }
      print "<IMG src='images/space.png' border='0'>\n";
      if ($display) {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=display&irq_num=$irq_num'>";
	print "<IMG src='images/nodata.png' border='0'>";
	print "</a>";
      } else {
	print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=display&irq_num=$irq_num'>";
	print "<IMG src='images/showdata.png' border='0'>";
	print "</a>";
      }
      print "<IMG src='images/space.png' border='0'>";
      print "<A href='edit_module.cgi?idx=$in{'idx'}&opt=rm&irq_num=$irq_num'>";
      print "<IMG src='images/trash.png' border='0'>";
      print "</a>";
    }
    print    "</td>\n";
    $irq_num++;
  }
  print "</table></td></tr>";

  if ($in{'opt'} eq 'edit') {
    local ($irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display)
      = split(',', $my_config->{'irq' . $in{'irq_num'}});

    print     "<tr>\n";
    print      "<td>";
    print       "<table border width='100%'>\n";
    print        "<tr $tb><td><b>" . $text{'module_irq'} . "</b></td></tr>\n";
    print        "<tr $cb>";
    print         "<td>";
    print          "<form action='save_config.cgi'>";
    print          "<table width='100%'>\n";
    print           "<tr>\n";
    print            "<td width='30%'><b>" . $text{'module_word_irq'} . "</b></td>\n";
    print             "<td nowrap><b>$irq_name</b></td>";
    print            "</tr>\n<tr>";
    print             "<td><b>" . $text{'word_colour'} . "</b></td>\n";
    print             "<td>";
    &option_colourselect('irq_colour', $irq_colour);
    print             "</td>";
    print            "</tr>\n<tr>";
    print            "<td><b>$text{'module_display_name'}</b></td><td>";
    printf "<input type='text' name='irq_name_def' size='9' maxlength='9' value='%s'>", $irq_name;
    print            "</td></tr>\n<tr>";
    print            "<td><b>" . $text{'word_display_data'} . "</b></td>";
    printf "<td><input type='radio' name='display_def' value='0' %s> $text{'word_yes'}  ",
      !$display ? 'checked' : '' ;
    printf "<input type='radio' name='display_def' value='1' %s> $text{'word_no'}</td></tr>\n",
      $display ? 'checked' : '';
    print            "<tr><td><b>" . $text{'module_pause_datacapture'} . "</b></td>";
    printf "<td><input type='radio' name='runstop_def' value='0' %s> $text{'word_yes'}  ",
      !$runstop ? 'checked' : '' ;
    printf "<input type=radio name='runstop_def' value='1' %s> $text{'word_no'}</td></tr>\n",
      $runstop ? 'checked' : '';
    print           "<tr>";
    print            "<td align='left'>";
    print              "<input type='hidden' name='idx' value='" . $in{'idx'} . "'>";
    print              "<input type='hidden' name='opt' value='update_irq'>";
    print              "<input type='hidden' name='irq' value='" . $in{'irq_num'} . "'>";
    print              "<input type='submit' value='Update:'>";
    print             "</form>";
    print            "</td>";
    print           "</tr>";
    print          "</table>";
    print         "</td>";
    print        "</tr>";
    print       "</table>";
    print      "</td>";
    print     "</tr>\n";
  } else {

    $i = 0;
    print     "<tr>\n";
    print      "<td>";
    print       "<table border width='100%'>\n";
    print        "<tr $tb><td><b>" . $text{'module_new_irq'} . "</b></td></tr>\n";
    print        "<tr $cb>";
    print         "<td>";
    print          "<form action='save_config.cgi'>";
    print          "<table width='100%'>\n";
    print           "<tr>\n";
    print            "<td width='30%'><b>" . $text{'module_word_irq'} . "</b></td>\n";
    print            "<td nowrap><b>";
    print              "<select name='new_irq_def'>\n";
    printf              "<option value='" . $i++ . "'>%s\n", $text{'word_select'};
    foreach $vlist (@irq_list) {
      printf            "<option value='" . $i++ . "'>%s\n", $vlist;
    }
    print              "</select>";
    print             "</b></td>";
    print            "</tr><tr>";
    print             "<td><b>" . $text{'word_colour'} . "</b></td>\n";
    print             "<td>";
    &option_colourselect('new_irq_colour', '');
    print             "</td>";
    print            "</tr>\n<tr>";
    print            "<td><b>$text{'module_display_name'}</b></td><td>";
    printf "<input type='text' name='new_irq_name_def' size='9' maxlength='9' value='%s'>", $irq_name;
    print            "</tr><tr>";
    print            "<td><b>" . $text{'word_display_data'} . "</b></td>";
    print "<td><input type='radio' name='new_display_def' value='1' checked> $text{'word_yes'}  ";
    print "<input type='radio' name='new_display_def' value='0'> $text{'word_no'}</td>";
    print           "</tr>\n";
    print           "<tr>";
    print            "<td align='left'>";
    print              "<input type='hidden' name='idx' value='" . $in{'idx'} . "'>";
    print              "<input type='hidden' name='opt' value='new_irq'>";
    print              "<input type='hidden' name='irq' value='" . $irq_num . "'>";
    print              "<input type='submit' value='Add New:'>";
    print             "</form>";
    print            "</td>";
    print           "</tr>";
    print          "</table>";
    print         "</td>";
    print        "</tr>";
    print       "</table>";
    print      "</td>";
    print     "</tr>\n";
  }
  print    "</table>";
  print   "</td>";
  print  "</tr>";
  print "</table>\n";
}

sub mod_config_save {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  if ($in{'opt'} eq 'new_irq') {
    if (($in{'new_irq_def'} eq '0') ||
	($in{'new_irq_name_def'} eq '') ||
	(($in{'new_irq_colour1_def'} eq 'nocolour') &&
	 ($in{'new_irq_colour2_def'} eq ''))) {
      &error($text{'save_einvaled_selection'});
    }

    if (!-r $minfo->{'dbtables'} . $my_config->{'db_uid'} . ".rrd") {
      local ($cmd);

      $cmd = $config{'rrdtool'};
      $cmd .= " create " . $minfo->{'dbtables'} . $my_config->{'db_uid'} . ".rrd";
      $cmd .= " --start 0 --step 60";
      $cmd .= " DS:irq:COUNTER:100:U:U";
      $cmd .= " RRA:MAX:0.5:1:70";
      $cmd .= " RRA:MAX:0.5:2:750";
      $cmd .= " RRA:MAX:0.5:15:650";
      $cmd .= " RRA:MAX:0.5:50:650";

      `$cmd`;
    }

    @irq_list = split ( /\n/, `cat /proc/interrupts | grep -v CPU | awk '{print \$1}'`);
    my $irq_number = @irq_list[$in{'new_irq_def'} - 1 ];

    $irq_number =~ s/://;

    $my_config->{'irq' . $in{'irq'}} =
      join(',', $my_config->{'db_uid'}++,
	   $irq_number,
	   $in{'new_irq_name_def'},
	   $in{'new_irq_colour2_def'} ? $in{'new_irq_colour2_def'} : $in{'new_irq_colour1_def'},
	   0, $in{'new_display_def'} ? '0' : '1'
	  );

    &write_config($minfo, $my_config, $defaults);
    &redirect("edit_module.cgi?idx=$in{'idx'}");
  } elsif ($in{'opt'} eq 'del_irq') {
    my $irq_num = $in{'irq'};

    local ($irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display)
      = split(',', $my_config->{'irq' . $in{'irq'}});

    while ($my_config->{'irq' . $irq_num} ne '') {
      $my_config->{'irq' . $irq_num} = $my_config->{'irq' . ($irq_num + 1)};
      $irq_num++;
    }

    $my_config->{'irq' . ($irq_num - 1)} = '';

    &write_config($minfo, $my_config, $defaults);

    unlink ($minfo->{'dbtables'} . $irq_id . '.rrd');
    &redirect("edit_module.cgi?idx=$in{'idx'}");
  } elsif ($in{'opt'} eq 'update_irq') {
    my $irq_num = $in{'irq'};

    if (($in{'irq_colour1_def'} eq 'nocolour') &&
	 ($in{'irq_colour2_def'} eq '')) {
      &error($text{'save_einvaled_selection'});
      exit;
    }

    local ($irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display)
      = split(',', $my_config->{'irq' . $in{'irq'}});

    $my_config->{'irq' . $in{'irq'}} =
      join(',', $irq_id,
	   $irq_number,
	   $in{'irq_name_def'},
	   $in{'irq_colour2_def'} ? $in{'irq_colour2_def'} : $in{'irq_colour1_def'},
	   $in{'runstop_def'}, $in{'display_def'}
	  );

    &write_config($minfo, $my_config, $defaults);
    &redirect("edit_module.cgi?idx=$in{'idx'}");
  }

  &error($in);
}

sub write_config {
  local ($minfo, $my_config, $defaults) = ($_[0], $_[1], $_[2]);

  local (%new_config, %sh_config);

  $new_config{'sample_rate'} = $my_config->{'sample_rate'};
  $sh_config{'sample_rate'} = $my_config->{'sample_rate'};
  $new_config{'runstop'} = $my_config->{'runstop'};
  $new_config{'db_uid'} = $my_config->{'db_uid'};

  my $i = 1;
  while ($my_config->{'irq' . $i} ne '') {
    $new_config{'irq' . $i} = $my_config->{'irq' . $i};
    $i++;
  }

  $i = 1;
  my $n = 1;
  while ($new_config{'irq' . $i} ne '') {
    local ($irq_id, $irq_number, $irq_name, $irq_colour, $runstop, $display)
      = split(',', $new_config{'irq' . $i++});

    if (!$runstop && !$new_config{'runstop'}) {
      $sh_config{'irq' . $n} = '(';
      $sh_config{'irq' . $n} .= $irq_number . ' ';
      $sh_config{'irq' . $n} .= $irq_id;
      $sh_config{'irq' . $n++} .= ')';
    }
  }

  unlink ($minfo->{'config_file'});
  unlink ($minfo->{'config_dir'} . '/irq.cfg');

  &write_file($minfo->{'config_file'}, \%new_config);
  &write_file($minfo->{'config_dir'} . '/irq.cfg', \%sh_config);
}

return 1;
